//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.StockForms
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			DateTime start = DateTime.Now.Date;
			DateTime end = start + new TimeSpan(30 * TimeSpan.TicksPerDay);

			// Set the date displayed in the Calendar
			_calendar.Date = start;
			_calendar.EndDate = end;

			// Create some initial events
			Appointment app = null;

			app = new Appointment();
			app.HeaderText = "My first appointment";
			app.StartTime = start + new TimeSpan(10 * TimeSpan.TicksPerDay);
			app.EndTime = start + new TimeSpan(12 * TimeSpan.TicksPerDay);
			_calendar.Schedule.Items.Add(app);

			app = new Appointment();
			app.HeaderText = "Another event";
			app.StartTime = start + new TimeSpan(11 * TimeSpan.TicksPerDay);
			app.EndTime = app.StartTime;
			_calendar.Schedule.Items.Add(app);

			// Recur every four days
			Recurrence recurrence = new Recurrence();
			recurrence.Pattern = RecurrencePattern.Daily;
			recurrence.DailyRecurrence = DailyRecurrence.ByDayInterval;
			recurrence.Days = 4;
			recurrence.RecurrenceEnd = RecurrenceEnd.Never;
			recurrence.StartDate = start + new TimeSpan(6 * TimeSpan.TicksPerDay);

			app = new Appointment();
			app.HeaderText = "Recurring event";
			app.StartTime = start;
			app.EndTime = start + TimeSpan.FromDays(1);
			app.Recurrence = recurrence;
			_calendar.Schedule.Items.Add(app);

			_calendar.Theme = ThemeType.Light;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _calendar_ItemClick(object sender, MindFusion.Scheduling.WinForms.ItemMouseEventArgs e)
		{
			if (e.Clicks == 2 && e.Button == MouseButtons.Left)
			{
				// Reset the drag operation. This
				// ensures that the item is not dragged or
				// in-place edited after the form is displayed
				_calendar.ResetDrag();

				// Get the clicked event
				Appointment app = e.Item as Appointment;

				// Check if it is a recurring event
				if (app.RecurrenceState == RecurrenceState.None)
				{
					// It is not a recurring event;
					// Display the form for it
					AppointmentForm form = new AppointmentForm(_calendar.Schedule);
					form.Text = app.HeaderText + " - Event";
					form.SetAppointment(app);

					DialogResult result = form.ShowDialog(this);
					switch (result)
					{

						case DialogResult.OK:
						{
							// The form updates the edited item,
							// we need only to invalidate
							_calendar.Invalidate();
						}
							break;

						case DialogResult.Abort:
						{
							// Abort is returned when the user
							// clicks the 'Delete' button
							if (MessageBox.Show("Are you sure you want to delete the event?",
								"Delete Event", MessageBoxButtons.YesNo) == DialogResult.Yes)
							{
								_calendar.Schedule.Items.Remove(app);
								_calendar.Invalidate();
							}
						}
							break;

					}
				}
				else
				{
					// It is a recurring event;
					// Ask the user whether he or she wants to
					// edited only this occurrence or the entire
					// recurrence pattern
					QForm qForm = new QForm();
					if (qForm.ShowDialog(this) == DialogResult.OK)
					{
						if (qForm.OpenOccurrence)
						{
							AppointmentForm form = new AppointmentForm(_calendar.Schedule);
							form.Text = app.HeaderText + " - Recurring Event";
							form.SetAppointment(app);

							DialogResult result = form.ShowDialog(this);
							switch (result)
							{

								case DialogResult.OK:
								{
									// The form updates the edited item,
									// we need only to invalidate
									_calendar.Invalidate();
								}
									break;

								case DialogResult.Abort:
								{
									// Abort is returned when the user
									// clicks the 'Delete' button.
									// Ask the user if he wants to delete
									// only this event or the series
									DForm dForm = new DForm();
									if (dForm.ShowDialog(this) == DialogResult.OK)
									{
										if (dForm.DeleteOccurrence)
										{
											// Mark the event as deleted
											app.Recurrence.MarkException(app, true);
											_calendar.Invalidate();
										}
										else if (dForm.DeleteSeries)
										{
											app.Recurrence = null;
											_calendar.Invalidate();
										}
									}
								}
									break;

							}
						}
						else if (qForm.OpenSeries)
						{
							AppointmentForm form = new AppointmentForm(_calendar.Schedule);
							form.Text = app.HeaderText + " - Recurring Event";

							// To edit the series, set the recurrence master
							// as the item being edited
							form.SetAppointment(app.Recurrence.Master as Appointment);

							DialogResult result = form.ShowDialog(this);
							switch (result)
							{

								case DialogResult.OK:
								{
									// The form updates the edited item,
									// we need only to invalidate
									_calendar.Invalidate();
								}
									break;

								case DialogResult.Abort:
								{
									// Abort is returned when the user
									// clicks the 'Delete' button.
									// Delete all items in the series by
									// removing the master
									_calendar.Schedule.Items.Remove(app.Recurrence.Master);
									_calendar.Invalidate();
								}
									break;

							}
						}
					}
				}
			}
		}

		private void _calendar_RecurringItemDeleting(object sender, MindFusion.Scheduling.WinForms.RecurringItemConfirmEventArgs e)
		{
			// Ask the user if he wants to delete the item
			// only or all occurrences in the series
			DForm dForm = new DForm();
			if (dForm.ShowDialog(this) == DialogResult.OK)
			{
				if (dForm.DeleteOccurrence)
				{
					e.ConfirmOccurrence = true;
				}
				else if (dForm.DeleteSeries)
				{
					e.ConfirmAll = true;
				}
			}
			else
			{
				e.ConfirmOccurrence = false;
			}
		}


		private string _appPath;
	}
}
